@@ -1,4 +1,3 @@
-
 #define EXTUNIX_WANT_ATFILE
 #include "config.h"
 
@@ -71,15 +70,16 @@ CAMLprim value caml_extunix_fstatat(value v_dirfd, value v_name, value v_flags)
 {
   CAMLparam3(v_dirfd, v_name, v_flags);
   int ret;
+  int dirfd = Int_val(v_dirfd);
   struct stat buf;
-  char* p = strdup(String_val(v_name));
+  char* p = caml_stat_strdup(String_val(v_name));
   int flags = caml_convert_flag_list(v_flags, at_flags_table);
   flags &= (AT_SYMLINK_NOFOLLOW | AT_NO_AUTOMOUNT); /* only allowed flags here */
 
   caml_enter_blocking_section();
-  ret = fstatat(Int_val(v_dirfd), p, &buf, flags);
+  ret = fstatat(dirfd, p, &buf, flags);
   caml_leave_blocking_section();
-  free(p);
+  caml_stat_free(p);
   if (ret != 0) uerror("fstatat", v_name);
   if (buf.st_size > Max_long && (buf.st_mode & S_IFMT) == S_IFREG)
     unix_error(EOVERFLOW, "fstatat", v_name);
@@ -89,15 +89,16 @@ CAMLprim value caml_extunix_fstatat(value v_dirfd, value v_name, value v_flags)
 CAMLprim value caml_extunix_unlinkat(value v_dirfd, value v_name, value v_flags)
 {
   CAMLparam3(v_dirfd, v_name, v_flags);
-  char* p = strdup(String_val(v_name));
+  int dirfd = Int_val(dirfd);
+  char* p = caml_stat_strdup(String_val(v_name));
   int ret = 0;
   int flags = caml_convert_flag_list(v_flags, at_flags_table);
   flags &= AT_REMOVEDIR;  /* only allowed flag here */
 
   caml_enter_blocking_section();
-  ret = unlinkat(Int_val(v_dirfd), p, flags);
+  ret = unlinkat(dirfd, p, flags);
   caml_leave_blocking_section();
-  free(p);
+  caml_stat_free(p);
   if (ret != 0) uerror("unlinkat", v_name);
   CAMLreturn(Val_unit);
 }
@@ -105,9 +106,14 @@ CAMLprim value caml_extunix_unlinkat(value v_dirfd, value v_name, value v_flags)
 CAMLprim value caml_extunix_renameat(value v_oldfd, value v_oldname, value v_newfd, value v_newname)
 {
   CAMLparam4(v_oldfd, v_oldname, v_newfd, v_newname);
+  int oldfd = Int_val(v_oldfd), newfd = Int_val(newfd);
+  char *oldname = caml_stat_strdup(String_val(v_oldname)),
+       *newname = caml_stat_strdup(String_val(v_newname));
   caml_enter_blocking_section();
-  int ret = renameat(Int_val(v_oldfd), String_val(v_oldname), Int_val(v_newfd), String_val(v_newname));
+  int ret = renameat(oldfd, oldname, newfd, newname);
   caml_leave_blocking_section();
+  caml_stat_free(newname);
+  caml_stat_free(oldname);
   if (ret != 0) uerror("renameat", v_oldname);
   CAMLreturn(Val_unit);
 }
@@ -115,9 +121,12 @@ CAMLprim value caml_extunix_renameat(value v_oldfd, value v_oldname, value v_new
 CAMLprim value caml_extunix_mkdirat(value v_dirfd, value v_name, value v_mode)
 {
   CAMLparam3(v_dirfd, v_name, v_mode);
+  int dirfd = Int_val(v_dirfd), mode = Int_val(v_mode);
+  char *name = caml_stat_strdup(String_val(v_name));
   caml_enter_blocking_section();
-  int ret = mkdirat(Int_val(v_dirfd), String_val(v_name), Int_val(v_mode));
+  int ret = mkdirat(dirfd, name, mode);
   caml_leave_blocking_section();
+  caml_stat_free(name);
   if (ret != 0) uerror("mkdirat", v_name);
   CAMLreturn(Val_unit);
 }
@@ -125,12 +134,17 @@ CAMLprim value caml_extunix_mkdirat(value v_dirfd, value v_name, value v_mode)
 CAMLprim value caml_extunix_linkat(value v_olddirfd, value v_oldname, value v_newdirfd, value v_newname, value v_flags)
 {
   CAMLparam5(v_olddirfd, v_oldname, v_newdirfd, v_newname, v_flags);
+  int olddirfd = Int_val(v_olddirfd), newdirfd = Int_val(v_newdirfd);
+  char *oldname = caml_stat_strdup(String_val(v_oldname)),
+       *newname = caml_stat_strdup(String_val(v_newname));
   int ret = 0;
   int flags = caml_convert_flag_list(v_flags, at_flags_table);
   flags &= AT_SYMLINK_FOLLOW;  /* only allowed flag here */
   caml_enter_blocking_section();
-  ret = linkat(Int_val(v_olddirfd), String_val(v_oldname), Int_val(v_newdirfd), String_val(v_newname), flags);
+  ret = linkat(olddirfd, oldname, newdirfd, newname, flags);
   caml_leave_blocking_section();
+  caml_stat_free(newname);
+  caml_stat_free(oldname);
   if (ret != 0) uerror("linkat", v_oldname);
   CAMLreturn(Val_unit);
 }
@@ -138,12 +152,15 @@ CAMLprim value caml_extunix_linkat(value v_olddirfd, value v_oldname, value v_ne
 CAMLprim value caml_extunix_fchownat(value v_dirfd, value v_name, value v_owner, value v_group, value v_flags)
 {
   CAMLparam5(v_dirfd, v_name, v_owner, v_group, v_flags);
+  int dirfd = Int_val(v_dirfd), owner = Int_val(v_owner), group = Int_val(v_group);
+  char *name = caml_stat_strdup(String_val(v_name));
   int ret = 0;
   int flags = caml_convert_flag_list(v_flags, at_flags_table);
   flags &= (AT_SYMLINK_NOFOLLOW /* | AT_EMPTY_PATH */);  /* only allowed flag here */
   caml_enter_blocking_section();
-  ret = fchownat(Int_val(v_dirfd), String_val(v_name), Int_val(v_owner), Int_val(v_group), flags);
+  ret = fchownat(dirfd, name, owner, group, flags);
   caml_leave_blocking_section();
+  caml_stat_free(name);
   if (ret != 0) uerror("fchownat", v_name);
   CAMLreturn(Val_unit);
 }
@@ -151,12 +168,15 @@ CAMLprim value caml_extunix_fchownat(value v_dirfd, value v_name, value v_owner,
 CAMLprim value caml_extunix_fchmodat(value v_dirfd, value v_name, value v_mode, value v_flags)
 {
   CAMLparam4(v_dirfd, v_name, v_mode, v_flags);
+  int dirfd = Int_val(v_dirfd), mode = Int_val(v_mode);
+  char *name = caml_stat_strdup(String_val(v_name));
   int ret = 0;
   int flags = caml_convert_flag_list(v_flags, at_flags_table);
   flags &= AT_SYMLINK_NOFOLLOW;  /* only allowed flag here */
   caml_enter_blocking_section();
-  ret = fchmodat(Int_val(v_dirfd), String_val(v_name), Int_val(v_mode), flags);
+  ret = fchmodat(dirfd, name, mode, flags);
   caml_leave_blocking_section();
+  caml_stat_free(name);
   if (ret != 0) uerror("fchmodat", v_name);
   CAMLreturn(Val_unit);
 }
@@ -164,27 +184,32 @@ CAMLprim value caml_extunix_fchmodat(value v_dirfd, value v_name, value v_mode,
 CAMLprim value caml_extunix_symlinkat(value v_path, value v_newdirfd, value v_newname)
 {
   CAMLparam3(v_path, v_newdirfd, v_newname);
+  char *path = caml_stat_strdup(String_val(v_path)),
+       *newname = caml_stat_strdup(String_val(v_newname));
+  int newdirfd = Int_val(v_newdirfd);
   caml_enter_blocking_section();
-  int ret = symlinkat(String_val(v_path), Int_val(v_newdirfd), String_val(v_newname));
+  int ret = symlinkat(path, newdirfd, newname);
   caml_leave_blocking_section();
+  caml_stat_free(newname);
+  caml_stat_free(path);
   if (ret != 0) uerror("symlinkat", v_path);
   CAMLreturn(Val_unit);
 }
 
-CAMLprim value caml_extunix_openat(value v_dirfd, value path, value flags, value perm)
+CAMLprim value caml_extunix_openat(value v_dirfd, value v_path, value flags, value v_perm)
 {
-  CAMLparam4(v_dirfd, path, flags, perm);
+  CAMLparam4(v_dirfd, v_path, flags, v_perm);
+  int dirfd = Int_val(v_dirfd), perm = Int_val(v_perm);
   int ret, cv_flags;
-  char * p;
+  char *path = caml_stat_strdup(String_val(v_path));;
 
   cv_flags = extunix_open_flags(flags);
-  p = strdup(String_val(path));
   /* open on a named FIFO can block (PR#1533) */
   caml_enter_blocking_section();
-  ret = openat(Int_val(v_dirfd), p, cv_flags, Int_val(perm));
+  ret = openat(dirfd, path, cv_flags, perm);
   caml_leave_blocking_section();
-  free(p);
-  if (ret == -1) uerror("openat", path);
+  caml_stat_free(path);
+  if (ret == -1) uerror("openat", v_path);
   CAMLreturn (Val_int(ret));
 }
 
@@ -197,17 +222,17 @@ char *readlinkat_malloc (int dirfd, const char *filename)
 
   while (1)
     {
-      tmp = (char *) realloc (buffer, size);
+      tmp = caml_stat_resize_noexc (buffer, size);
       if (tmp == NULL)
       {
-        free(buffer); /* if failed, dealloc is not performed */
+        caml_stat_free (buffer); /* if failed, dealloc is not performed */
         return NULL;
       }
       buffer = tmp;
       nchars = readlinkat (dirfd, filename, buffer, size);
       if (nchars < 0)
       {
-          free (buffer);
+          caml_stat_free (buffer);
           return NULL;
       }
       if (nchars < size)
@@ -223,18 +248,18 @@ CAMLprim value caml_extunix_readlinkat(value v_dirfd, value v_name)
 {
   CAMLparam2(v_dirfd, v_name);
   CAMLlocal1(v_link);
+  int dirfd = Int_val(v_dirfd);
   char* res;
-  char* p = strdup(String_val(v_name));
+  char* name = caml_stat_strdup(String_val(v_name));
 
   caml_enter_blocking_section();
-  res = readlinkat_malloc(Int_val(v_dirfd), p);
+  res = readlinkat_malloc(dirfd, name);
   caml_leave_blocking_section();
-  free(p);
+  caml_stat_free(name);
   if (res == NULL) uerror("readlinkat", v_name);
   v_link = caml_copy_string(res);
-  free(res);
+  caml_stat_free(res);
   CAMLreturn(v_link);
 }
 
 #endif
-
